@@ -57,7 +57,7 @@ pub mod bw6_761;
 
 pub mod edwards_bls12;
 
-pub mod edwards_sw6;
+pub mod edwards_bw6;
 
 pub mod errors;
 pub use errors::*;
